/**
 * \file gre/autolabelparms.h
 * \brief GRE::AUTOLABELPARMS definitions
 *
 * \if NODOC
 * $Id: autolabelparms.h_v 1.3 2012/06/01 14:45:28 mju Exp $
 *
 * $Log: autolabelparms.h_v $
 * Revision 1.3  2012/06/01 14:45:28  mju
 * Add m_oldSelectMode for use in reading old layouts.
 *
 * Revision 1.2  2012/05/25 21:05:24  mju
 * Make members private and use enums.
 *
 * Revision 1.1  2012/05/25 17:14:18  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_AUTOLABELPARMS_H
#define	INC_GRE_AUTOLABELPARMS_H

#ifndef	INC_MI32_LABELFRAMEPARMS_H
#include <mi32/labelframeparms.h>
#endif

#ifndef	INC_RVC_OBJECTNAME_H
#include <rvc/objectname.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class DBASE;
	}
#endif

namespace GRE {

//===================================================================================================================

//! Automatic labeling parameters
struct AUTOLABELPARMS {
	public:

		//! How to obtain text.
		enum TEXTMODE {
			TEXTMODE_None =		0,		//!< No text, typically means no label at all
			TEXTMODE_ByField =	1,		//!< Text from field
			TEXTMODE_ByScript =	2		//!< Text from script
			};

		//! Possible places for "point" labels.
		enum PLACE_POINT {
			PLACE_POINT_None =			0,		//!< Don't label point features
			PLACE_POINT_AboveLeft =		1,
			PLACE_POINT_AboveCenter =	2,
			PLACE_POINT_AboveRight =	3,
			PLACE_POINT_CenterLeft =	4,
			PLACE_POINT_Center =			5,
			PLACE_POINT_CenterRight =	6,
			PLACE_POINT_BelowLeft =		7,
			PLACE_POINT_BelowCenter =	8,
			PLACE_POINT_BelowRight =	9,
			PLACE_POINT_LAST =			9
			};

		//! Placement for line labels.
		enum PLACE_LINE {
			PLACE_LINE_None =						0,		//!< Don't label line features
			PLACE_LINE_AlongOffsetAbove =		1,		//!< Along line, text "above"
			PLACE_LINE_AlongCenter =			2,		//!< Along line, text centered on line
			PLACE_LINE_AlongOffsetBelow =		3,		//!< Along line, text "below"
			PLACE_LINE_PerpendicularBefore =	4,		//!< Perpendicular to line, text "before"
			PLACE_LINE_PerpendicularCenter =	5,		//!< Perpendicular to line and centered on
			PLACE_LINE_PerpendicularAfter =	6,		//!< Perpendicular to line, text "after"
			};

		//! Placement for area labels.
		enum PLACE_AREA {
			PLACE_AREA_None =											0,	//!< Don't label area features
			PLACE_AREA_InsideIfFitElseNone =						1,	//!< Inside area or no label if won't fit
			PLACE_AREA_InsideIfFitElseCenter =					2,	//!< Inside area or center over area if won't fit
			PLACE_AREA_InsideIfFitElseOutsideWithLeader =	3,	//!< Inside area or outside with leader
			PLACE_AREA_InsideIfFitElseOutsideNoLeader =		4	//!< Inside area or outside with no leader
			};

		//! How to obtain priority when resolving conflicts.
		enum PRIORITYMODE {
			PRIORITYMODE_AllSame =	0,
			PRIORITYMODE_ByField =	1,
			PRIORITYMODE_ByScript =	2
			};

		enum OPTIMIZATION {
			OPTIMIZATION_None =			0x00,
			OPTIMIZATION_Fast =			0x01,
			OPTIMIZATION_Full =			0x02,
			OPTIMIZATION_HideOverlap =	0x04,
			};

		//! Get serializer item definition array.
		static const SERIALIZERITEM * SerialGetItemDef ();

		//! Default constructor
		AUTOLABELPARMS ();

		//! Copy constructor
		AUTOLABELPARMS (
			const AUTOLABELPARMS& rhs
			);

		//! Destructor
		~AUTOLABELPARMS ();

		//! Assignment
		AUTOLABELPARMS& operator= (
			const AUTOLABELPARMS& rhs
			);

		//! Access frame parameters for direct updating.
		LABELFRAMEPARMS& AccessFrameParms ()
			{ return (m_FrameParms); }

		RVC::OBJECTNAME& AccessPriorityFieldName ()
			{ return (m_PriorityFieldName); }

		RVC::OBJECTNAME& AccessPriorityTableName ()
			{ return (m_PriorityTableName); }

		RVC::OBJECTNAME& AccessTextFieldName ()
			{ return (m_TextFieldName); }

		//! Access text style for direct updating.
		TEXTSTYLE& AccessTextStyle ()
			{ return (m_TextStyle); }

		RVC::OBJECTNAME& AccessTextTableName ()
			{ return (m_TextTableName); }

		//! Check and validate for specified database and optionally fix mode if none valid.
		void CheckTable (
			const RVC::DBASE& db,					//!< Database object
			bool fixmode								//!< Fix mode if necessary
			);

		const LABELFRAMEPARMS& GetFrameParms () const
			{ return (m_FrameParms); }

		OPTIMIZATION GetOptimization () const
			{ return (m_Optimization); }

		//! Get place for "area" labels.
		PLACE_AREA GetPlaceArea () const
			{ return (m_PlaceArea); }

		//! Get place for "line" labels.
		PLACE_LINE GetPlaceLine () const
			{ return (m_PlaceLine); }

		//! Get place for "point" labels.
		PLACE_POINT GetPlacePoint () const
			{ return (m_PlacePoint); }

		const RVC::OBJECTNAME& GetPriorityFieldName () const
			{ return (m_PriorityFieldName); }

		const RVC::OBJECTNAME& GetPriorityTableName () const
			{ return (m_PriorityTableName); }

		PRIORITYMODE GetPriorityMode () const
			{ return (m_PriorityMode); }

		double GetScaleRangeMax () const
			{ return (m_MaxScaleVisible); }

		double GetScaleRangeMin () const
			{ return (m_MinScaleVisible); }

		const MISTRING& GetScript () const
			{ return (m_ScriptStr); }

		const RVC::OBJECTNAME& GetTextFieldName () const
			{ return (m_TextFieldName); }

		const RVC::OBJECTNAME& GetTextTableName () const
			{ return (m_TextTableName); }

		//! Get mode for label text.
		TEXTMODE GetTextMode () const
			{ return (m_TextMode); }

		const TEXTSTYLE& GetTextStyle () const
			{ return (m_TextStyle); }

		//! Determine if labels are selected and visible at specified scale.
		bool IsSelected (
			double scale								//!< Map scale
			) const { return (m_TextMode != TEXTMODE_None && scale >= m_MinScaleVisible && (m_MaxScaleVisible <= 0.0 || scale <= m_MaxScaleVisible)); }

		void SetFrameParms (
			const LABELFRAMEPARMS& FrameParms
			) { m_FrameParms = FrameParms; }

		void SetOptimization (
			OPTIMIZATION Optimization
			) { m_Optimization = Optimization; }

		//! Set place for "area" labels.
		void SetPlaceArea (
			PLACE_AREA PlaceArea
			) { m_PlaceArea = PlaceArea; }

		//! Set place for "line" labels.
		void SetPlaceLine (
			PLACE_LINE PlaceLine
			) { m_PlaceLine = PlaceLine; }

		//! Set place for "point" labels.
		void SetPlacePoint (
			PLACE_POINT PlacePoint
			) { m_PlacePoint = PlacePoint; }

		void SetPriorityField (
			const RVC::OBJECTNAME& TableName,
			const RVC::OBJECTNAME& FieldName
			) { m_PriorityTableName = TableName; m_PriorityFieldName = FieldName; }

		void SetPriorityMode (
			PRIORITYMODE PriorityMode
			) { m_PriorityMode = PriorityMode; }

		void SetScaleRange (
			double MinScaleVisible,
			double MaxScaleVisible
			) { m_MinScaleVisible = MinScaleVisible; m_MaxScaleVisible = MaxScaleVisible; }

		void SetScript (
			const MISTRING& ScriptStr
			) { m_ScriptStr = ScriptStr; }

		void SetTextField (
			const RVC::OBJECTNAME& TableName,
			const RVC::OBJECTNAME& FieldName
			) { m_TextTableName = TableName; m_TextFieldName = FieldName; }

		//! Set mode for label text.
		void SetTextMode (
			TEXTMODE TextMode
			) { m_TextMode = TextMode; }

		void SetTextStyle (
			const TEXTSTYLE& TextStyle
			) { m_TextStyle = TextStyle; }

		//! Validate and correct any illegal values.
		void Validate ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef AUTOLABELPARMS THISCLASS;

		TEXTMODE m_TextMode;
		PLACE_POINT m_PlacePoint;
		PLACE_LINE m_PlaceLine;
		PLACE_AREA m_PlaceArea;
		PRIORITYMODE m_PriorityMode;
		OPTIMIZATION m_Optimization;
		UINT8 m_OldSelectMode;
		UINT8 m_OldPosn;
		RVC::OBJECTNAME m_TextTableName;
		RVC::OBJECTNAME m_TextFieldName;
		MISTRING m_ScriptStr;
		TEXTSTYLE m_TextStyle;
		LABELFRAMEPARMS m_FrameParms;
		RVC::OBJECTNAME m_PriorityTableName;
		RVC::OBJECTNAME m_PriorityFieldName;
		double m_MinScaleVisible;
		double m_MaxScaleVisible;

		void Copy (const AUTOLABELPARMS& rhs);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(AUTOLABELPARMS::OPTIMIZATION)
#endif

//===================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_AUTOLABELPARMS_H
