/**
 * \file objectbase.h <gre/objectbase.h>
 * \brief Define base GRE object class
 *
 * \if NODOC
 * $Id: objectbase.h_v 1.12 2008/02/18 18:13:12 mju Exp $
 *
 * $Log: objectbase.h_v $
 * Revision 1.12  2008/02/18 18:13:12  mju
 * Make getTypeName const.
 *
 * Revision 1.11  2007/10/04 21:20:46  mju
 * Exclude macro from docs.
 *
 * Revision 1.10  2006/10/10 13:46:47  mju
 * Remove inline and micon hdr.
 *
 * Revision 1.9  2006/10/10 13:13:54  mju
 * Use miconstruct instead of typedef.
 *
 * Revision 1.8  2005/11/04 18:52:58  mju
 * Add object::type enum.
 *
 * Revision 1.7  2005/10/19 19:51:43  mju
 * Don't use old gre names.
 *
 * Revision 1.6  2005/10/17 15:11:15  mju
 * In setPrivPtr do range check on index.
 *
 * Revision 1.5  2005/10/14 17:32:40  mju
 * GRE namespace.
 *
 * Revision 1.4  2005/09/28 21:52:00  mju
 * Add is/setTemporary.
 *
 * Revision 1.3  2003/09/15 13:48:59  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.2  2003/07/30 15:33:20  mju
 * Ignore private sections.
 *
 * Revision 1.1  2003/06/16 19:40:04  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_OBJECTBASE_H
#define	INC_GRE_OBJECTBASE_H

#ifndef	INC_GRE_MSG_H
	#include <gre/msg.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
struct MICONSTRUCT;
#endif


namespace GRE {
//===================================================================================================================
//! Base class for other GRE "spatial" objects.
class OBJECT {

	public:

		//! GRE object types.
		enum TYPE {
			TYPE_None =		0,
			TYPE_System =	0x01,
			TYPE_Layout =	0x02,
			TYPE_Group =	0x04,
			TYPE_Layer =	0x08,
			TYPE_View =		0x10,
			TYPE_All =		0xFF
			};

		typedef void (*CBFUNC)(const GRE::MSG*, void*);

		//! Increment reference count.
		void AddRef (
			) { ++m_RefCount; }

		//! Allocate index into private pointer array to attach to.
		//! @see GetPrivPtr, SetPrivPtr
		//! It is often desirable for a process or library module to attach "private"
		//! data to one or more GRE objects.  This can be done using AllocPrivIndex()
		//! to allocate an entry into each object's private pointer array.  This method
		//! should only be called the first time the private index is needed.  It is
		//! the programmer's responsibility to retain this private index value.  Once
		//! a private index is allocated, use SetPrivPtr() to attach data to any GRE
		//! object and GetPrivPtr() to retrieve previously attached data.
		static int AllocPrivIndex (
			);

		//! Constructor
		explicit OBJECT (
			OBJECT::TYPE type								//!< Object type to construct
			);

		//! Destructor.
		virtual ~OBJECT (
			) = 0;

		//! Add callback function to object.
		//!
		//! The new callback function will be added to the head of the list.
		//!
		//! Callbacks may be added to any GRE object.  When a callback occurs for a particular object
		//! the message is sent to that object's callback list and any "ancestor" object's callback
		//! lists as well.  A callback will be called if the object type for the message was included in
		//! the object types specified when the callback was added.  This often eliminates the need to
		//! add callbacks to every GRE object which is created.  If a program creates a single group
		//! and needs information about the group and layers within it then a callback can be added
		//! to the group with OBJECT::TYPE_Group and OBJECT::TYPE_Layer specified.  Then within the
		//! callback itself a determination of the object type may be done.  Alternatively two callbacks
		//! might be used, one specifying OBJECT::TYPE_Group and the other specifying OBJECT::TYPE_Layer.
		//! Both callbacks would be added to the group, but no check of the object type would be needed
		//! within the callback since only one object type was specified when each callback was added.
		void CallbackAdd (
			CBFUNC cbfunc,									//!< Callback function
			void *cbdata,									//!< "Hook" data to pass to callback
			OBJECT::TYPE objtypes						//!< Object types for which messages are requested
			) { m_CallbackList.Add(cbfunc,cbdata,objtypes); }

		//! Invoke callbacks on object and its ancestors.
		//! This method is generally only invoked from within the GRE library itself.
		void CallbackInvoke (
			GRE::MSG::ACTION action,
			GRE::VIEW *view = 0,
			ELEMTYPE elemtype = ELEMTYPE_Empty
			) { GRE::MSG msg(action,this,view,elemtype); CallbackInvoke(msg); }
			
		//! Invoke callbacks on object and its ancestors.
		//! This method is generally only invoked from within the GRE library itself.
		void CallbackInvoke (
			GRE::MSG::ACTION action,
			ELEMTYPE elemtype
			) { GRE::MSG msg(action,this,elemtype); CallbackInvoke(msg); }
			
		//! Invoke callbacks on object and its ancestors.
		//! This method is generally only invoked from within the GRE library itself.
		void CallbackInvoke (
			GRE::VIEW *view,
			GRE::MSG::ACTION action
			) { GRE::MSG msg(action,this,view); CallbackInvoke(msg); }
			
		//! Invoke callbacks on object and its ancestors.
		//! This method is generally only invoked from within the GRE library itself.
		void CallbackInvoke (
			const GRE::MSG& msg
			);
		
		//! Remove callback function.
		//! All callbacks are automatically removed when the object is destroyed.
		//! However, if the data specified when the callback was added is freed 
		//! before the object is destroyed, all callbacks referencing that data
		//! should be removed.  In this case, the cbfunc may be specified as NULL
		//! and the cbdata should be the pointer about to be freed.  This will
		//! remove all callbacks referencing that pointer in a single step.
		void CallbackRemove (
			CBFUNC cbfunc,									//!< Callback function, matching CallbackAdd, or NULL to match all
			void *cbdata,									//!< Callback data, matching CallbackAdd
			OBJECT::TYPE objtypes = TYPE_All			//!< Object types
			) { m_CallbackList.Remove(cbfunc,cbdata,objtypes); }

		//! Request destruction of this object.
		//! @return true if destroyed, false if destruction deferred.
		//! If an object is dynamically created then unless the destruction occurs
		//! in the same function as creation the Destroy() method should be used 
		//! instead.  This is especially true if destruction is to be performed within
		//! a GRE callback.  Using delete on an object withing a callback may result
		//! in unpredictable behavior.
		bool Destroy (
			bool NotifyParent = true					//!< Notify parent of child's destruction if appropriate
			) { return (v_Destroy(NotifyParent)); }

		//! Determine if destruction has been requested for this object.
		//! Sometimes a callback may be invoked on an object after destruction has been
		//! requested on that object.  This may occur if destruction occurs in a callback
		//! and more callbacks are in the list after the one destroying the object.  In
		//! order to avoid unwanted processing or errors it may be necessary to determine
		//! if an object has been requested to be destroyed.  This is only useful within
		//! a callback since once the callbacks are completed the destruction will finish
		//! and thus the object will no longer exist.
		bool DestroyRequested (
			) const { return (m_DestroyRequested); }

		//! Get icon for this object type.
		MICONSTRUCT* GetIcon (
			) const { return (v_GetIcon()); }

		//! Get "private" pointer for this object.
		//! @see AllocPrivIndex, SetPrivPtr
		void *GetPrivPtr (
			int idx											//!< Private index obtained from AllocPrivIndex()
			) const { return (m_PrivData[idx]); }

		//! Get GRE object type
		OBJECT::TYPE GetType (
			) const { return (m_objtype); }

		//! Get name of this object type.
		//! @return Pointer to constant string.
		const char* GetTypeName (
			) const { return (v_GetTypeName()); }

		//! Get name of this object type in string.
		//! @return Pointer to string provided.
		const char* GetTypeName (
			char *string									//!< String to fill in
			) const;

		//! Determine if object in process of being destroyed.
		//! Sometimes a callback may be invoked on an object after destruction has begun
		//! on that object. In order to avoid unwanted processing or errors it may be
		//! necessary to determine if an object is in the process of being destroyed.
		//! This is only useful within a callback since once the callbacks are completed
		//! the destruction will finish and thus the object will no longer exist.
		bool IsBeingDestroyed (
			) const { return (m_BeingDestroyed); }

		//! Determine if object is considered 'temporary'.
		//! Temporary objects should not be saved or shown in list/treeview interface.
		bool IsTemporary (
			) const { return (m_IsTemporary); }

		//! Decrement reference count, perform deferred destruction if requested and no remaining references.
		void Release (
			);

		//! Set "private" pointer for this object.
		//! When an object is destroyed, the private pointers are NOT freed by the GRE
		//! system.  Thus it is the programmers responsibility to see that allocated data
		//! attached to an object is freed at the appropriate time, usually by checking for
		//! ACTION_Destroy in the callback message.
		//! @see GetPrivPtr, AllocPrivIndex
		void SetPrivPtr (
			int idx,											//!< Private index obtained from AllocPrivIndex()
			void *privptr									//!< Pointer value to set.
			) { if (idx > 0 && idx < s_PrivDataCount) m_PrivData[idx] = privptr; }

	protected:

		//! Determine if callbacks are currently being invoked on this object.
		bool CallbacksBeingInvoked (
			) const { return (m_RefCount > 0 || m_CallbackList.BeingInvoked()); }

		//! Notify object that is being destroyed, usually called by Destroy() method
		void SetBeingDestroyed (
			) { m_BeingDestroyed = true; }

		//! Request destruction of object, usually called by Destroy() method
		void SetDestroyRequest (
			) { m_DestroyRequested = true; }

		//! Set object as temporary.
		void SetTemporary (
			) { m_IsTemporary = true; }

	private:

		#ifndef GENERATING_DOXYGEN_OUTPUT

		// GRE::OBJECT::CBFUNCLIST
		//! Callback function list.
		class CBFUNCLIST {
			public:

				CBFUNCLIST (
					);

				~CBFUNCLIST (
					);

				//! Add function to callback list
				void Add (
					CBFUNC cbfunc,							//!< Function to add
					void *cbdata,							//!< "Hook" data to pass to function
					OBJECT::TYPE objtypes				//!< Object type(s) for which messages are requested
					);

				//! Determine if callback list is currently being invoked.
				bool BeingInvoked (
					) const { return (m_callcount > 0); }

				//! Invoke callbacks in list with specified message.
				void Invoke (
					const GRE::MSG& msg				//!< Message
					);

				//! Remove function from callback list
				void Remove (
					CBFUNC cbfunc,							//!< Function to remove, NULL to match all
					void *cbdata,							//!< "Hook" data, must match value used in Add, or NULL to match all
					OBJECT::TYPE objtypes = TYPE_All	//!< Object type(s) to match
					);

			private:
				// GRE::OBJECT::CBFUNCLIST::ITEM
				class ITEM {
					public:

						ITEM (
							CBFUNC cbfunc,
							void *cbdata,
							OBJECT::TYPE objtypes
							);

						CBFUNC m_cbfunc;
						void *m_cbdata;
						OBJECT::TYPE m_objtypes;
						ITEM *m_next;
					};

				ITEM *m_first;
				UINT16 m_callcount;
				bool m_DidRemove;
			};

		static int s_PrivDataCount;				//!< Number of private data index entries used
		static int s_PrivDataAlloc;				//!< Number of private data index entries allocated

		const TYPE m_objtype;						//!< Object type
		void **m_PrivData;							//!< Private data for modules to attach stuff to
		CBFUNCLIST m_CallbackList;					//!< Callback list
		bool m_DestroyRequested;					//!< Destroy requested while in callback
		bool m_BeingDestroyed;						//!< Object in process of being destroyed
		bool m_IsTemporary;							//!< Object is temporary, don't show in list/treeview
		UINT32 m_RefCount;							//!< Reference count for deferred destruction

		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Overridables.
		virtual bool v_Destroy (bool NotifyParent) = 0;
		virtual MICONSTRUCT* v_GetIcon () const = 0;
		virtual const char* v_GetTypeName () const = 0;
	};
#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(OBJECT::TYPE);
#endif

//===================================================================================================================
}	// End namespace GRE



#endif	// INC_GRE_OBJECTBASE_H
