/**
 * \file mgui/popup.h
 * \brief MGUI::POPUP definitions
 *
 * \if NODOC
 * $Id: popup.h_v 1.3 2011/08/24 15:09:34 mju Exp $
 *
 * $Log: popup.h_v $
 * Revision 1.3  2011/08/24 15:09:34  mju
 * Make ctrl parm in open const.
 *
 * Revision 1.2  2011/08/22 20:27:15  mju
 * Add getDlgParent.
 *
 * Revision 1.1  2011/08/19 20:32:00  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_POPUP_H
#define	INC_MGUI_POPUP_H

#ifndef	INC_MGUI_LAYOUT_H
#include <mgui/layout.h>
#endif

#if !defined(INC_MI32_XBASE_H) && defined(X_NATIVE)
#include <mi32/xbase.h>
#endif

namespace MGUI {

//=====================================================================================================================
//! Base class for 'popup' windows.
//! A popup window has no title bar and minimal borders.
//! It is generally considered to be transient and appears in response to user clicking on another control such as a button.
//! Clicking outside the popup window will close it, returning EUserCancel from Open().
//! Usually clicking on a control in the popup will perform some action and also close the window.
//! A popup cannot be used as a parent for other dialogs.
class POPUP {
	public:

		enum LOCATION {
			LOCATION_Xmin =		0x00,		//!< Minimum X (left)
			LOCATION_Xmax =		0x01,		//!< Maximum X (right)
			LOCATION_Xcenter =	0x02,
			LOCATION_Ymin =		0x00,		//!< Minimum Y (top)
			LOCATION_Ymax =		0x04,		//!< Maximum Y (bottom)
			LOCATION_Ycenter =	0x08,
			LOCATION_XminYmin =	(LOCATION_Xmin|LOCATION_Ymin),
			LOCATION_XminYmax =	(LOCATION_Xmin|LOCATION_Ymax),
			LOCATION_X =			0x03,
			LOCATION_Y =			0x0C,
			};

		//! Constructor.
		POPUP ();

		//! Destructor.
		virtual ~POPUP ();

		//! Create popup over specified parent.
		ERRVALUE Create (
			MDLGPARENT dlgparent				//!< Parent for popup
			);
			
		//! Destroy the shell.
		void Destroy ();

		//! Determine if has been 'created' yet.
		bool IsCreated (
			) const {
			return (m_shell != 0);
			}

		//! Open the popup window relative to specified control and wait for it to be closed.
		//! @return EUserCancel if user closes by clicking outside window.
		//! If popup has not been created yet will use specified control's shell as parent to create popup.
		ERRVALUE Open (
			const MGUI::CTRL& ctrl,								//!< Control to show relative to
			const LRECT2D *pRect = 0,							//!< Rectangle in specified control coordinates, 0 to use control's entire rectangle
			int width = 0,											//!< Width override, 0 to retain current width
			int height = 0,										//!< Height override, 0 to retain current height
			LOCATION locInCtrl = LOCATION_XminYmax,		//!< Location on control / rectangle
			LOCATION locInPopup = LOCATION_XminYmin		//!< Location in popup to match
			);

		//! Open the popup window relative to specified point and wait for it to be closed.
		//! @return EUserCancel if user closes by clicking outside window.
		//! Create() must have been performed before using this variant of Open().
		ERRVALUE Open (
			MGUI::POINT pt,										//!< Point in display coordinates
			int width = 0,											//!< Width override, 0 to retain current width
			int height = 0,										//!< Height override, 0 to retain current height
			LOCATION locInPopup = LOCATION_XminYmin		//!< Location in popup to place at specified point
			);

	protected:

		//! Close the popup window and return specified value.
		void Close (ERRVALUE RetValue);

		//! Get parent for use in launching other dialogs from popup.
		MDLGPARENT GetDlgParent (
			) const { return (m_dlgparent); }

		//! Get main layout pane.
		//! Typically used in v_CreateContent.
		LAYOUT_PANE_BASE& GetMainPane (
			) { return (m_MainPane); }

		//! Set return value if previously closed.
		//! Has no effect if Close was not previously called.
		//! Typically used if a dialog was launched from the popup.
		void SetReturnValue (
			ERRVALUE RetValue
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		Widget m_shell;
		MDLGPARENT m_dlgparent;
		LAYOUT_PANE_SHELL m_MainPane;
		ERRVALUE m_RetValue;

		static void OnNotifyXEvent (Widget, POPUP*, XEvent*, Boolean*);

		// Unimplemented.
		POPUP (const POPUP&);
		POPUP& operator= (const POPUP&);

		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables

		//! Create contents.
		//! Derived class must implement this.  Return 0 for success or an error code < 0.
		//! This will be called by the popup creation code and must not be called directly.
		//! This is normally overridden only by the specific popup implementation and not by the
		//! code that actually uses that popup implementation.  Implementation should call
		//! GetMainPane() to determine the pane to create the controls in.
		//! This function should not open another dialog, modal or non-modal.
		virtual ERRVALUE v_CreateContent () = 0;

		//! Called when popup opened.
		//! Default implementation does nothing.
		virtual void v_OnOpen ();

	};

#ifndef	GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(POPUP::LOCATION)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_POPUP_H
