/**
 * \file mi32/mgps.h
 * \brief Definitions for MicroImages GPS library
 *
 * \if NODOC
 * $Id: mgps.h_v 1.62 2011/09/06 21:13:58 mju Exp $
 *
 * $Log: mgps.h_v $
 * Revision 1.62  2011/09/06 21:13:58  mju
 * Exclude macro from dox.
 *
 * Revision 1.61  2008/12/09 14:09:44  mju
 * Add private methods.
 *
 * Revision 1.60  2008/12/08 23:53:27  mju
 * Add pfileRawLog member.
 *
 * Revision 1.59  2008/12/08 21:20:36  mju
 * Add status_error.
 *
 * Revision 1.58  2007/03/22 22:14:06  dwilliss
 * use new PORT class
 *
 * Revision 1.57  2007/03/07 21:59:30  bcolombe
 * Added DlgCreateImport and ReadLogText for reading a formatted text log file
 *
 * Revision 1.56  2007/02/28 22:45:25  bcolombe
 * Added support for GPX xml log files
 *
 * Revision 1.55  2007/01/17 15:01:59  mju
 * Add dtor for observer.
 *
 * Revision 1.54  2006/11/10 20:51:12  bcolombe
 * Added Garmin support
 *
 * Revision 1.53  2006/10/13 20:45:44  mju
 * Remove rvcdefns hdr.
 *
 * Revision 1.52  2005/12/22 22:30:25  mju
 * Use file_text in logfile.
 *
 * Revision 1.51  2005/12/22 22:14:14  mju
 * Use file_text for recording.
 *
 * Revision 1.50  2005/06/27 20:55:55  mju
 * Fix virtual call.
 *
 * Revision 1.49  2005/06/27 17:27:40  mju
 * Change to use MGUI for both X and MFC.
 *
 * Revision 1.48  2005/03/31 16:57:17  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.47  2004/06/29 22:02:48  mju
 * Use coordrefsys.
 *
 * Revision 1.44  2003/10/28 17:36:10  dwilliss
 * Added a buffer for storing partial message between polls
 *
 * Revision 1.40  2002/05/14 20:05:11  dwilliss
 * Make SOURCEMGR a friend of SOURCE so it can make SOURCE::OBSERVERMGR a
 * friend of itself.
 *
 * Revision 1.30  2001/07/24 17:23:25  mju
 * Redesign to use observer pattern.
 * Put everything in MGPS namespace.
 * Begin adding support for native Windows.
 *
 * Revision 1.1  1997/12/30  23:14:37  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_MGPS_H
#define	INC_MI32_MGPS_H

#ifdef USE_MGUI

#ifndef	INC_MI32_OBSERVER_H
#include <mi32/observer.h>
#endif

#ifndef	INC_MI32_SPATREF_H
#include <mi32/spatref.h>
#endif

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifndef	INC_MI32_COLOR_H
#include <mi32/color.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

#ifndef INC_MI32_XML_H
#include <mi32/xml.h>
#endif

#ifndef	INC_RVC_OBJECTNAME_H
#include <rvc/objectname.h>
#endif

#ifndef	INC_MGUI_TIMER_H
#include <mgui/timer.h>
#endif

#include <mi32/platformsupport.h>

//-------------------------------------------------------------------------------------------------------------------
//		Forward declarations

#ifndef GENERATING_DOXYGEN_OUTPUT
class PORT;
struct POINTSTYLE;
class FILE_TEXT;
namespace MGUI {
	class LAYOUT_PANE_BASE;
	class FORM;
	}
namespace MGD {
	class CONTEXT;
	}
namespace RVC {
	class STYLE;
	}
#endif


namespace MGPS {

class SOURCEMGR;

//-------------------------------------------------------------------------------------------------------------------

//! Individual GPS position report.
struct REPORT {

	enum DATASRC {
		DATASRC_GPS2D =		0,
		DATASRC_GPS3D =		1,
		DATASRC_DGPS2D =		2,
		DATASRC_DGPS3D =		3,
		DATASRC_DR =			6,
		DATASRC_DegradedDR =	8,
		DATASRC_Unknown =		9
		};

	enum VALID {
		VALID_None =			0x0000,
		VALID_PositionXY =	0x0001,
		VALID_PositionZ =		0x0002,
		VALID_VelocityXY =	0x0004,
		VALID_VelocityZ =		0x0008,
		VALID_Heading =		0x0010,
		VALID_Speed =			0x0020,
		VALID_NumSat =			0x0040,
		VALID_DateTime =		0x0080
		};

	DPOINT3D Position;						//!< x=Longitude degrees, y=Latitude degrees, z=Elevation meters
	DPOINT3D Velocity;						//!< x=East, y=North, z=Up, all in meters/second
	double Heading;							//!< Degrees from True North increasing eastwardly
	double Speed;								//!< Speed over Ground
	DATASRC DataSrc;							//!< Data source
	VALID Valid;								//!< Flags for which values are valid
	UINT16 Year;								//!< Year (4-digit)
	UINT8 Month;								//!< Month (1-12)
	UINT8 Day;									//!< Day (1-31)
	UINT8 Hour;									//!< Hour (0-23)
	UINT8 Minute;								//!< Minute (0-59)
	UINT8 Second;								//!< Second (0-59)
	UINT8 NumSatellites;						//!< Number of satellites used
	};
#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(REPORT::VALID)
#endif

//-------------------------------------------------------------------------------------------------------------------

//! GPS symbol.
class SYMBOL {
	public:

		enum FLAGS {
			FLAG_None =					0,
			FLAG_PatternSymbol =		0x00000001,	//!< Use "symbol" pattern
			FLAG_PatternBitmap =		0x00000002,	//!< Use bitmap pattern
			FLAG_SizeBySpeed =		0x00000004,	//!< Control symbol size by speed
			FLAG_SizeLengthOnly =	0x00000008,	//!< Only resize "length"
			FLAG_KeepUpright =		0x00000010	//!< Keep left/right oriented symbols "upright"
			};

		enum ORIENT {
			ORIENT_HeadingUp =		0,
			ORIENT_HeadingRight =	1,
			ORIENT_HeadingDown =		2,
			ORIENT_HeadingLeft =		3
			};

		//! Default constructor.
		SYMBOL (
			);

		//! Copy constructor.
		SYMBOL (
			const SYMBOL& rhs
			);

		//! Destructor.
		~SYMBOL (
			);

		//! Assignment
		SYMBOL& operator= (
			const SYMBOL& rhs
			);

		//! Draw.
		bool Draw (
			MGD::CONTEXT* ggc,
			const DPOINT2D* point,
			const DPOINT2D* dirpoint,
			const DPOINT2D* waypoint,
			LRECT2D* retextents
			) const;

		//! Get symbol color.
		const COLOR& GetColor (
			) const { return (m_Color); }

		//! Get built-in pattern number.
		UINT8 GetPatternNum (
			) const { return (m_PatternNum); }

		//! Get size in millimeters.
		double GetSize (
			) const { return (m_Size); }

		//! Retrieve symbol style.
		ERRVALUE GetStyle (
			RVC::STYLE& ObjStyle,				//!< Open style object
			POINTSTYLE& Style						//!< Style definition returned
			) const;

		//! Read symbol settings from ini file.
		void ReadIniSettings (
			const char* IniGroup,
			const char* SymbolName,
			RVC::STYLE& ObjStyle					//!< Open style object
			);

		//! Read pattern from style object.
		void ReadPattern (  
			RVC::STYLE& ObjStyle					//!< Open style object
			);

		//! Set the color.
		void SetColor (
			const COLOR& color
			) { m_Color = color; }

		//! Set color for symbol by name.
		void SetColor (
			const char *name						//!< Color name
			) { m_Color.SetByName(name); }

		//! Set the pattern number.
		void SetPatternNum (
			UINT8 PatternNum
			) { m_PatternNum = PatternNum; }

		//! Set the size.
		void SetSize (
			double size
			) { m_Size = static_cast<float>(size); }

		//! Set symbol from POINTSTYLE.
		ERRVALUE SetStyle (
			RVC::STYLE& ObjStyle,				//!< Open style object
			const POINTSTYLE& Style				//!< Style to set
			);

		//! Write settings to ini file.
		void WriteIniSettings (
			const char* IniGroup,
			const char* SymbolName
			) const;

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT
		UINT8 m_PatternNum;						//!< Pattern number if built-in
		ORIENT m_Orient;							//!< Orientation mode
		float m_Size;								//!< Symbol size in millimeters if built-in
		float m_Angle;								//!< Symbol rotation angle
		FLAGS m_Flags;
		COLOR m_Color;								//!< Symbol color
		RVC::OBJECTNAME m_PatternName;		//!< Name of pattern if not built-in
		int m_PatternSize;						//!< Size of pattern in bytes if not built-in
		void *m_Pattern;							//!< Pattern if not built-in symbol
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(SYMBOL::FLAGS)
#endif


//-------------------------------------------------------------------------------------------------------------------

//! Base class for GPS Source.
class SOURCE {
	public:

		enum STATUS {
			STATUS_Good =				0,			//!< Good signal, data available
			STATUS_NoResponse =		1,		 	//!< No response from device
			STATUS_Initializing =	2,		 	//!< Initializing
			STATUS_Acquiring =		3,		 	//!< Acquiring signal from satellites
			STATUS_SignalLost =		4,		 	//!< Lost signal from satellites
			STATUS_EndOfLogFile =	5,		 	//!< End of log file
			STATUS_Error =				6
			};

		enum SYMBOLTYPE {
			SYMBOLTYPE_Stopped =	0,			 	//!< Symbol to show while stopped
			SYMBOLTYPE_Moving =	1			 	//!< Symbol to show while moving
			};

		class OBSERVER;
		class DLG;

		//! Observer class to derive from to be notified of events on this source.
		class OBSERVER : public OBSERVER_BASE {
			public:

				//! Constructor.
				OBSERVER (
					SOURCE *source
					): OBSERVER_BASE(source->m_ObserverMgr) { }

				//! Destructor.
				virtual ~OBSERVER ();

				//! Notify that new source was added.
				void NotifyAdd (
					SOURCE *source
					) { v_OnAdd(source); }

				//! Notify that source has been 'closed'.
				void NotifyClose (
					SOURCE *source
					) { v_OnClose(source); }

				//! Notify that source has been 'opened'.
				void NotifyOpen (
					SOURCE *source
					) { v_OnOpen(source); }

				//! Notify that source has been polled for new position.
				void NotifyPoll (
					const SOURCE *source,
					const REPORT& report
					) { v_OnPoll(source,report); }

				//! Notify that source about to be removed from manager.
				void NotifyRemove (
					SOURCE *source
					) { v_OnRemove(source); }

			private:

				//! Called when new source added.
				virtual void v_OnAdd (
					SOURCE *source
					);

				//! Called when source has been 'closed'.
				virtual void v_OnClose (
					SOURCE *source
					);

				//! Called when source has been 'opened'.
				virtual void v_OnOpen (
					SOURCE *source
					);

				//! Called when source has been polled for new position.
				virtual void v_OnPoll (
					const SOURCE *source,
					const REPORT& report
					);

				//! Called when source about to be removed from manager.
				virtual void v_OnRemove (
					SOURCE *source
					);
			};

		//! Constructor.
		SOURCE (  
			);

		//! Destructor.
		virtual ~SOURCE ( 
			);

		void Close (
			);

		//! Close control dialog for this source.
		void CloseControlDialog (
			);

	#ifdef USE_MGUI
		//! Create "form" for source controls.
		MGUI::FORM* CreateControlForm (
			MGUI::LAYOUT_PANE_BASE& parentpane
			) { return (v_CreateControlForm(parentpane)); }
	#endif

		//! Delete source definition from .ini file.
		void DeleteFromIni (
			) const { v_DeleteFromIni(); }

		const SPATREF::COORDREFSYS& GetCoordRefSys (
			) const { return (m_CoordRefSys); }

		const char* GetIniGroupKey (
			) const { return (m_IniGroup); }

		//! Get name of source.
		//! @return Name string, do not free.
		const MISTRING& GetName (
			) const { return (m_name); }

		//! Get current polling interval.
		//! @return Polling interval in seconds.
		double GetPollInterval (
			) const { return (m_PollInterval); }

		//! Get symbol definition.
		//! @return Symbol.
		const SYMBOL& GetSymbol (
			SYMBOLTYPE symboltype				//!< Symbol type
			) const {
			return ((symboltype == SYMBOLTYPE_Stopped) ? m_SymbolStopped : m_SymbolMoving);
			}

		//! Get last GPS position.
		//! @return True if have data, false if not.
		bool GetLastData (
			REPORT* gpsdata						//!< Position information returned
			) const {
			if (m_HasData) {
				*gpsdata = m_LastData;
				}
			return (m_HasData);
			}

		//! Get current source status.
		//! @return Status.
		STATUS GetStatus (
			) const { return (m_LastStatus); }

		//! Determine if source is a "device" or not.
		bool IsDevice (
			) const { return (v_IsDevice()); }

		//! Determine if source is "managed".
		//! @return True if managed, false if not.
		bool IsManaged (
			) const { return (m_IsManaged); }

		//! Determine if source is "open".
		//! @return True if open, false if not.
		bool IsOpen (
			) const { return (m_IsOpen); }

		//! Place source under control of "source manager".
		int Manage (
			);

		//! Open source.
		ERRVALUE Open (  
			);

		//! Open control dialog for source.
		void OpenControlDialog (
			);

		//! Read next available value.
		int ReadNext (
			) { return (v_ReadNext()); }

		//! Set name of source.
		void SetName (
			const MISTRING& name				  	//!< New name
			) { m_name = name; }

		//! Set polling interval.
		void SetPollInterval (
			double PollInterval					//!< New polling interval in seconds
			);

		//! Set new symbol.
		void SetSymbol (
			SYMBOLTYPE symboltype,	  			//!< Symbol type
			const SYMBOL& symbol,			//!< New symbol definition
			bool UpdateIni				  			//!< Set if should update Ini settings
			);

		//! Unmanage source.
		void Unmanage (
			);

	protected:

		//! Clear last GPS position.
		void ClearLastData (
			) { m_HasData = false; }

		//! Get parent for dialogs.
		MDLGPARENT GetDlgParent (
			) const { return (m_DlgParent); }

		//! See if has current GPS position.
		bool HasData (
			) const { return (m_HasData); }

		//! Read user settings for source from Ini file.
		void ReadIniSettings (
			const char* IniGroup
			);

		//! Set current GPS data.
		void SetCurrentData (
			const REPORT& gpsdata
			) { m_HasData = true; m_LastData = gpsdata; }

		//! Set the current source status.
		void SetCurrentStatus (
			STATUS status
			) { m_LastStatus = status; }

		void SetDlgParent (
			MDLGPARENT dlgparent
			) { m_DlgParent = dlgparent; }

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		class OBSERVERMGR : public SUBJECT<OBSERVER> {
			public:
				void NotifyAdd (SOURCE *source);
				void NotifyClose (SOURCE *source);
				void NotifyOpen (SOURCE *source);
				void NotifyPoll (const SOURCE *source, const REPORT& report);
				void NotifyRemove (SOURCE *source);
			};

		OBSERVERMGR m_ObserverMgr;
		MISTRING m_name;							// Name of source
		char m_IniGroup[16];						// Ini group for source settings if device
		bool m_IsOpen;
		bool m_IsManaged;
		bool m_HasData;
		STATUS m_LastStatus;
		SYMBOL m_SymbolStopped;
		SYMBOL m_SymbolMoving;
		double m_PollInterval;					// Polling interval in seconds
		SPATREF::COORDREFSYS m_CoordRefSys;
		REPORT m_LastData;
		MDLGPARENT m_DlgParent;					// Used as parent for file open, etc...
		DLG *m_pDlg;								// Control dialog

	#ifdef USE_MGUI
		MGUI::TIMER m_Timer;
		void OnTimer ();
	#endif

		SOURCE (const SOURCE&);
		SOURCE& operator= (const SOURCE&);

		friend class MGPS::SOURCE::OBSERVER;
		friend class MGPS::SOURCE::DLG;
		friend class MGPS::SOURCEMGR;
	#endif // GENERATING_DOXYGEN_OUTPUT

		// SOURCE Overridables

		//! Close source.
		virtual void v_Close (
			);

	#ifdef USE_MGUI
		//! Create "form" for source controls.
		virtual MGUI::FORM* v_CreateControlForm (
			MGUI::LAYOUT_PANE_BASE& parentpane
			) = 0;
	#endif

		//! Delete source definition from .ini file.
		virtual void v_DeleteFromIni (
			) const;

		//! Determine if source is a "device" or not.
		virtual bool v_IsDevice (
			) const;

		//! Open source.
		virtual int v_Open (  
			) = 0;

		//! Read next available value.
		virtual int v_ReadNext (
			) = 0;

	};


//-------------------------------------------------------------------------------------------------------------------

//! GPS source device.
class SOURCE_DEVICE : public SOURCE {
	public:

		//! Device protocol.
		enum PROTOCOL {
			PROTOCOL_NoDevice = 0,				//!< No protocol set yet
			PROTOCOL_NMEA = 1,					//!< NMEA 0183
			PROTOCOL_TAIP = 2,						//!< Trimble ASCII
			PROTOCOL_GARMIN = 3
			};

		//! Output log format
		enum FORMAT {
			FORMAT_GPX = 0,						//!< GPS Exchange (GPX) xml log
			FORMAT_MI= 1						//!< MicroImages log format
			};

		//! Device hardware parameters.
		class HWPARM {
			public:

				//! Constructor.
				HWPARM (
					);

				ERRVALUE DlgEditSettings (
					MDLGPARENT dlgparent
					);

				const MISTRING& GetDeviceDesc (
					) const { return (m_devicedesc); }

				int ReadIni (
					INIHANDLE IniHandle,
					UINT16 DeviceNum
					);

				int ReadIni (
					UINT16 DeviceNum
					);

				void SetPortName (
					const MISTRING& portname
					) { m_portname = portname; }

				void SetPortParm (
					const char *portparm
					) { strcpy(m_portparm,portparm); }

				void SetPortType (
					UINT16 porttypenum
					) { m_porttypenum = porttypenum; }

				void SetProtocol (
					PROTOCOL protocol
					) { m_protocol = protocol; }

				void WriteIni (
					INIHANDLE IniHandle
					) const;

				ERRVALUE WriteIni (
					);

			private:
   			#ifndef GENERATING_DOXYGEN_OUTPUT
				class DLG;
				friend class HWPARM::DLG;

				UINT16 m_devicenum;						// GPS device number
				UINT16 m_porttypenum;					// PORTTYPE_...
				PROTOCOL m_protocol;						// Protocol
				MISTRING m_portname;						// Port name
				char m_portparm[80];						// Port parameters
				MISTRING m_devicedesc;					// GPS device description
				double m_PollInterval;					// Polling interval in seconds

				friend class MGPS::SOURCE_DEVICE;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Create and manage default GPS source devices as defined in Ini file.
		static void CreateDefault ();

		//! Constructor.
		SOURCE_DEVICE (
			const HWPARM& portparm
			);

		//! Destructor.
		virtual ~SOURCE_DEVICE (
			);

		//! Get the device number.
		int GetDeviceNum (
			) const { return (m_hwparm.m_devicenum); }

		//! Begin recording to specified log file.
		ERRVALUE RecordStart (
			const FILEPATH& filepath
			);

		//! Stop recording.
		void RecordStop (
			);

		//! Get output log format
		FORMAT GetFormat() const {return m_format;}

		//! Set output log format
		void SetFormat(FORMAT format) {m_format = format;}

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		HWPARM m_hwparm;
		PORT *m_port;
		XMLDOC *m_xmldoc;
		FILE_TEXT *m_pFileRecord;
		FILE_TEXT *m_pFileRawLog;
		FILEPATH m_filepath;
		time_t m_lastmsgtime;
		FORMAT m_format;
		char m_tempbuf[100];		// For storage of partial message

		void OpenRawLogFile ();
		int ReadChar (char& c, int failvalue = -3155);	/* EGPS_CantReadMsg */
		int ReadStr (char *str, int len);
		int ReadMsgNMEA (char *datastr);
		int ReadMsgTAIP (const char *msgid, char *datastr);
		int ReadNMEA (REPORT*);
		int ReadTAIP (REPORT*);

		#ifdef PLATFORM_SUPPORTS_GARMIN
		int ReadGARMIN (REPORT*);
		#endif

		SOURCE_DEVICE (const SOURCE_DEVICE&);
		SOURCE_DEVICE& operator= (const SOURCE_DEVICE&);

		// SOURCE overrides.
		virtual void v_Close ();
	#ifdef USE_MGUI
		virtual MGUI::FORM* v_CreateControlForm (MGUI::LAYOUT_PANE_BASE& parentpane);
	#endif
		virtual void v_DeleteFromIni () const;
		virtual bool v_IsDevice () const;
		virtual int v_Open ();
		virtual int v_ReadNext ();

		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//-------------------------------------------------------------------------------------------------------------------

//! GPS log file source.
class SOURCE_LOGFILE : public SOURCE {
	public:

		//! Log format.
		enum FORMAT {
			FORMAT_Automatic,						//!< Automatically determine format if possible
			FORMAT_MicroImages,					//!< MicroImages log format
			FORMAT_NMEA,							//!< Raw NMEA 0183 messages
			FORMAT_Satlok,							//!< Satlok ag equipment
			FORMAT_GPX,								//!< GPS exchange (gpx) xml log file
			FORMAT_Text								//!< Log imported from text file
			};

		//! Direction for playback.
		enum DIRECTION {
			DIRECTION_Forward,
			DIRECTION_Reverse
			};

		//! Dialog to prompt user for logfile and create class instance.
		static ERRVALUE DlgCreate (
			MDLGPARENT dlgparent
			);

		//! Dialog for setting import from formatted text parameters
		static ERRVALUE DlgCreateImport (
			MDLGPARENT dlgparent
			);

		//! Constructor.
		SOURCE_LOGFILE (
			const FILEPATH& filepath,
			FORMAT format = FORMAT_Automatic
			);

		//! Destructor.
		virtual ~SOURCE_LOGFILE (
			);

		DIRECTION GetDirection (
			) const { return (m_Direction); }

		//! Get current playback rate.
		//! @return Relative rate.
		double GetPlaybackRate (
			) const { return (m_PlaybackRate); }

		bool IsAtStart (
			) const { return (m_CurPosition <= 0); }

		//! Get "paused" state.
		bool IsPaused (
			) const { return (m_IsPaused); }

		//! Set auto-repeat mode.
		void SetAutoRepeat (
			bool AutoRepeat = true
			) { m_AutoRepeat = AutoRepeat; }

		//! Set playback direction.
		void SetDirection (
			DIRECTION direction					//!< Direction to set
			) { m_Direction = direction; }

		//! Set paused state.
		void SetPaused (
			bool Paused = true					//!< True to pause, false to resume
			) { m_IsPaused = Paused; }

		//! Set current position within log file.
		void SetPosition (
			double position  						//!< Position, 0.0 for start
			);

		//! Set relative playback rate.
		void SetPlaybackRate (
			double rate					 		//!< Rate, 1.0 for "realtime"
			);

		//! Skip forward to next section.
		void SkipForward (
			);

		//! Skip backward to previous section.
		void SkipReverse (  
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		enum PLAYBTN {
			PLAYBTN_Stop = 0,
			PLAYBTN_Pause = 1,
			PLAYBTN_FastRev = 2,
			PLAYBTN_Reverse = 3,
			PLAYBTN_Forward = 4,
			PLAYBTN_FastFwd = 5,
			PLAYBTN_COUNT
			};

		class LOGPOINT {
			public:
				DPOINT2D Position;						//!< x = Longitude, y = Latitude degrees
				float Time;									//!< Time in seconds from start of log
				float Elevation;							//!< Elevation in meters
				float Heading;								//!< Degrees from True North increasing eastwardly
				float Speed;								//!< Speed over ground
				float VelocityZ;							//!< Z Velocity
				UINT8 DataSource;
				UINT8 NumSatellites;						//!< Number of satellites used for point
				UINT16 ValidFlags;						//!< Flags for which values are valid
			};

		class SECTIONINFO {
			public:
				float m_StartTime;						//!< Relative start time for section
				float m_EndTime;							//!< Relative end time for section
				int m_FirstPoint;							//!< First point in section
				int m_NumPoints;							//!< Number of points in sction
			};

		FILEPATH m_filepath;
		FORMAT m_Format;
		int m_NumPoints;
		LOGPOINT *m_LogPoints;
		int m_NumSections;
		SECTIONINFO *m_SectionInfo;
		double m_BaseTime;
		float m_TotalTime;
		float m_CurPosition;
		int m_CurSection;
		int m_CurPoint;
		bool m_AutoRepeat;
		bool m_AtSectionEnd;
		bool m_IsPaused;
		double m_PlaybackRate;
		DIRECTION m_Direction;

		int AddLogPoint (
			LOGPOINT& logpoint,
			int& PointsAlloc,
			int logday,
			double logtime
			);

		int ReadLogGPX();

		int ReadLogText();

		int ReadLogMI (
			FILE_TEXT& file
			);

		int ReadLogNMEA (
			FILE_TEXT& file
			);

		int ReadLogSatlok (
			FILE_TEXT& file
			);

		SOURCE_LOGFILE (const SOURCE_LOGFILE&);
		SOURCE_LOGFILE& operator= (const SOURCE_LOGFILE&);

		// SOURCE overrides.
		virtual void v_Close ();
	#ifdef USE_MGUI
		virtual MGUI::FORM* v_CreateControlForm (MGUI::LAYOUT_PANE_BASE& parentpane);
	#endif
		virtual int v_Open ();
		virtual int v_ReadNext ();

		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//-------------------------------------------------------------------------------------------------------------------

typedef MILIST<SOURCE*> SOURCELIST;

//! GPS Source Manager.
class SOURCEMGR {
	public:

		class OBSERVER;

		//! Observer class to derive from to be notified of source manager events.
		class OBSERVER : public OBSERVER_BASE {
			public:

				//! Constructor.
				OBSERVER (
					): OBSERVER_BASE(SOURCEMGR::s_ObserverMgr) { }

				//! Destructor.
				virtual ~OBSERVER ();

				//! Called when new source added.
				virtual void v_OnAdd (
					SOURCE *source
					);

				//! Called when source has been 'closed'.
				virtual void v_OnClose (
					SOURCE *source
					);

				//! Called when source has been 'opened'.
				virtual void v_OnOpen (
					SOURCE *source
					);

				//! Called when source has been polled for new position.
				virtual void v_OnPoll (
					const SOURCE *source,
					const REPORT& report
					);

				//! Called when source about to be removed from manager.
				virtual void v_OnRemove (
					SOURCE *source
					);
			};

		static void DestroyAllSources (
			);

		static int GetNumSources (
			) { return (s_SourceList.GetNumItems()); }

		static const SOURCELIST& GetSourceList (
			) { return (s_SourceList); }

		static ERRVALUE OpenDialog (
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		class OBSERVERMGR : public SUBJECT<OBSERVER> {
			public:
				void NotifyAdd (SOURCE *source);
				void NotifyClose (SOURCE *source);
				void NotifyOpen (SOURCE *source);
				void NotifyPoll (const SOURCE *source, const REPORT& report);
				void NotifyRemove (SOURCE *source);
			};

		class DLG;

		static OBSERVERMGR s_ObserverMgr;
		static SOURCELIST s_SourceList;
		static DLG *s_pDlg;

		static void Add (SOURCE *source);
		static void Remove (SOURCE *source);

		//! Unimplemented.
		SOURCEMGR (const SOURCEMGR&);
		SOURCEMGR& operator= (const SOURCEMGR&);

	friend class MGPS::SOURCE;
	friend class MGPS::SOURCE::OBSERVERMGR;
	friend class MGPS::SOURCEMGR::OBSERVER;
	#endif // GENERATING_DOXYGEN_OUTPUT
	};


//-------------------------------------------------------------------------------------------------------------------

}	// End namespace MGPS.

#endif	// USE_MGUI

#endif	// INC_MI32_MGPS_H
