/**
 * \file micache.h <mi32/micache.h>
 * \brief MICACHE class definition.
 *
 * \if NODOC
 * $Id: micachearray.h_v 1.1 2008/06/17 22:03:13 scowan Exp $
 *
 * $Log: micachearray.h_v $
 * Revision 1.1  2008/06/17 22:03:13  scowan
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MICACHEARRAY_H
#define	INC_MI32_MICACHEARRAY_H

#ifndef	INC_MI32_MICACHE_H
#include <mi32/micache.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
template <class _CT> class MICACHEARRAYIO;
#endif //!< GENERATING_DOXYGEN_OUTPUT

template <class _CT>
class MICACHEARRAY : private MICACHEIO {
	public:
		//! Default constructor.
		//! This performs initialization only.  Use the appropriate Create() method to actually
		//! create the cache.
		explicit MICACHEARRAY (MICACHEARRAYIO<_CT>& ioref) : m_ioref(ioref), m_ItemSize(0), m_NumItemsInCacheEntry(0) {}
		
		//! Destructor.
		virtual ~MICACHEARRAY () {}
	
		//! Create cache with equal-sized entries.
		//! Allocation will be handled by the MICACHE class.  If items are only to be read
		//! and not modified then the CACHEIO.CacheSave() method does not need to do anything but return 0.
		//! 'NumCache' needs to be at least as much as the number of threads that will use the cache, 
		//! otherwise an ECacheItemsExhausted error may result.
		ERRVALUE Create (
			INT32 NumBuckets,							//!< Number of cache buckets to use.
			INT32 BucketSize = 32768,				//!< Initial size of the bucket in bytes
			bool bSynchronizeAccess = false,		//!< Set to 'true' if multiple threads will use this cache
			INT32 ItemSize = sizeof(_CT)			//!< Size of each array item in bytes
			) {
			m_ItemSize = ItemSize;
			m_NumItemsInCacheEntry = BucketSize / ItemSize;
			return (m_Cache.Create(NumBuckets, m_NumItemsInCacheEntry * ItemSize, *this, 0, bSynchronizeAccess));
			}
	
		//! Flush all modified entries to external storage.
		//! This may cause the WriteFunc callback to be invoked.
		ERRVALUE Flush (
			) { return (m_Cache.Flush()); }
			
		//! Free the cache.
		void Free (
			) { m_Cache.Free(); }

		//! Read data from cache item.
		//! An EInvalidCacheResize error will be returned if this method is called while 
		//! another thread is calling Resize().
		ERRVALUE Read (
			INT32 ItemIdx,							//!< Item number to read
			_CT& item,								//!< Item to read into
			void* UserDataPtr = 0				//!< User data pointer to attach to the cache item before reading
			) { 
			INT32 ItemID = ItemIdx / m_NumItemsInCacheEntry;
			INT32 Offset = (ItemIdx % m_NumItemsInCacheEntry) * m_ItemSize;
			return (m_Cache.Read(ItemID, Offset, m_ItemSize, &item, UserDataPtr));
			}

		//! Write data to cache item.
		//! An EInvalidCacheResize error will be returned if this method is called while 
		//! another thread is calling Resize().
		ERRVALUE Write (
			INT32 ItemIdx,							//!< Item number to write
			const _CT& item,						//!< Item to write from
			void* UserDataPtr = 0				//!< User data pointer to attach to the cache item before writing
			) { 
			INT32 ItemID = ItemIdx / m_NumItemsInCacheEntry;
			INT32 Offset = (ItemIdx % m_NumItemsInCacheEntry) * m_ItemSize;
			return (m_Cache.Write(ItemID, Offset, m_ItemSize, &item, UserDataPtr));
			}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MICACHE m_Cache;
		MICACHEARRAYIO<_CT>& m_ioref;
		UINT32 m_ItemSize;
		UINT32 m_NumItemsInCacheEntry;
		
		ERRVALUE CacheLoad (
			MICACHE::ITEMINFO& iteminfo
			) { return (m_ioref.ArrayLoad(iteminfo.GetItemID() * m_NumItemsInCacheEntry, m_ItemSize, m_NumItemsInCacheEntry, static_cast<_CT*>(iteminfo.GetMemPtr()))); }
			
		ERRVALUE CacheSave (
			MICACHE::ITEMINFO& iteminfo
			) { return (m_ioref.ArraySave(iteminfo.GetItemID() * m_NumItemsInCacheEntry, m_ItemSize, m_NumItemsInCacheEntry, static_cast<const _CT*>(iteminfo.GetMemPtr()))); }
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


//! Interface class for providing access to external storage.
template <class _CT>
class MICACHEARRAYIO {
	protected:
		//! Load cache entry from external storage.
		//! If program is handling allocation this must allocate memory for item and set using MICACHE::ITEMINFO::SetMemPtr().
		virtual ERRVALUE ArrayLoad (
			INT32 ItemIdx,
			INT32 ItemSize,		//!< Unless changed in MICACHEARRAY::Create() this will be sizeof(_CT)
			INT32 NumItems,
			_CT* BucketPtr 
			) = 0;

		//! Save cache entry to external storage.
		//! If program is handling allocation this must free any allocated memory used by item.
		virtual ERRVALUE ArraySave (
			INT32 ItemIdx,
			INT32 ItemSize,		//!< Unless changed in MICACHEARRAY::Create() this will be sizeof(_CT)
			INT32 NumItems,
			const _CT* BucketPtr
			) = 0;
	friend class MICACHEARRAY<_CT>;
	};

#endif	// INC_MI32_MICACHEARRAY_H

