/**
 * \file <mi32/milog.h>
 * \brief MILOG definitions
 *
 * \if NODOC
 * $Id: milog.h_v 1.8 2011/04/12 17:11:24 mju Exp $
 *
 * $Log: milog.h_v $
 * Revision 1.8  2011/04/12 17:11:24  mju
 * Change mode to onlyInternal and add noChkUserEnabled.
 *
 * Revision 1.7  2011/03/31 19:34:32  mju
 * Support mode_OnlyUserEnabled.
 *
 * Revision 1.6  2011/03/31 17:27:50  mju
 * Improve doc comments.
 *
 * Revision 1.5  2011/03/31 17:10:48  mju
 * Add mode_AllowClient, remove mode_OnlyInternal.
 *
 * Revision 1.4  2011/03/31 16:49:02  mju
 * Add mode_OnlyDV.
 *
 * Revision 1.3  2011/03/31 15:43:23  mju
 * Add setLabel.
 *
 * Revision 1.2  2011/03/30 21:22:37  mju
 * Add isActive.
 *
 * Revision 1.1  2011/03/30 20:01:53  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_MILOG_H
#define	INC_MI32_MILOG_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

// Forward declarations.
class MISTRING;
class FILEPATH;

#ifdef MISYSTEMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

//=====================================================================================================================

//! Class to manage logging to memory or file.
//! Provides control over whether logging is done based on license or debug.
//! This class is thread-safe in that text may be added to the log from threads.  However, the log may not be
//! created or retrieved from while there is a possibility that it may be updated from a thread.
class CLASSLIBEXPORT MILOG {
	public:

		//! Logging modes.
		enum MODE {
			MODE_NoLog =				0x0000,	//!< Don't do any logging
			MODE_Memory =				0x0001,	//!< Log to memory string (specific to log instance)
			MODE_OwnFile =				0x0002,	//!< Log to own (process instance) file
			MODE_SharedFile =			0x0003,	//!< Log to shared (session) file
			MODE_OnlyInternal =		0x0010,	//!< Only log for MicroImages "internal" license
			MODE_OnlyProgrammer =	0x0020,	//!< Only log for "programmer" key
			MODE_OnlyDebug =			0x0040,	//!< Only log in debug build
			MODE_OnlyDV =				0x0080,	//!< Only log in "development" version
			MODE_NoChkUserEnabled =	0x0100,	//!< Don't check if user has enabled "extended logging" - use with caution
			MODE_MASK_Location =		0x000F
			};

		//! Constructor.
		MILOG ();

		//! Destructor.
		~MILOG ();

		//! Add text to log (thread-safe).
		void AddText (const char* text);

		//! Add text to log (thread-safe).
		void AddText (const MISTRING& text);

		//! Create log, will clear if in memory.
		//! If MODE_OwnFile is specified, a new file will be created based on the current timestamp and process name.
		//! By default, logging is only done if the user turns on "extended logging".  Logging may be further limited
		//! based on the MODE setting.  Logging on should be used only when necessary to avoid degrading performance.
		ERRVALUE Create (
			MODE mode,
			INT32 MaxLogsMemory = 0		//!< Maximum number of logs (if MODE_OwnFile) or memory size (if MODE_Memory)
			);

		//! Get path to log file if was created with MODE_OwnFile.
		//! The process-instance file name is based on the application "name" and the time of the first call to
		//! Create() with MODE_OwnFile specified.  Thus, the same file is shared for all logging within a specific
		//! process instance.
		const FILEPATH& GetOwnFilePath () const;

		//! Get log string if in memory.
		//! If logging is to memory, each MILOG instance will have its own string.
		const MISTRING& GetString () const;

		//! Determine if logging is active.
		bool IsActive () const
			{ return (m_pPriv != 0); }

		//! Set label to include in log.
		//! Label will appear after timestamp in log.
		//! Log must have been created first.
		void SetLabel (
			const char *label				//!< Label to include, NULL for none
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		// Disallowed.
		MILOG (const MILOG&);
		MILOG& operator= (const MILOG&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(MILOG::MODE)
#endif

//=====================================================================================================================

#undef	CLASSLIBEXPORT

#endif
