/**
 * \file statmultivar.h <mi32/statmultivar.h>
 * \brief STAT_MULTIVAR class definitions.
 *
 * \if NODOC
 * $Id: statmultivar.h_v 1.3 2012/12/28 16:19:07 mju Exp $
 *
 * $Log: statmultivar.h_v $
 * Revision 1.3  2012/12/28 16:19:07  mju
 * Add copy ctor/op= and add method.
 * Remove unused totalVariance member.
 *
 * Revision 1.2  2006/01/26 15:56:29  mju
 * Add computeMultilinearRegression.
 *
 * Revision 1.1  2005/08/16 15:18:54  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_STATMULTIVAR_H
#define	INC_MI32_STATMULTIVAR_H

#ifndef	INC_MI32_MIMATRIX_H
#include <mi32/mimatrix.h>
#endif


//! Class for accumulating and computing multivariate statistics.
class STAT_MULTIVAR {
	public:

		//! Default constructor.
		STAT_MULTIVAR ();

		//! Copy constructor.
		STAT_MULTIVAR (
			const STAT_MULTIVAR& rhs
			);

		//! Destructor.
		~STAT_MULTIVAR ();

		//! Assignment operator.
		STAT_MULTIVAR& operator= (
			const STAT_MULTIVAR& rhs
			);

		//! Accumulate values for single sample.
		ERRVALUE Accumulate (
			const double *values				//!< Array of values
			);

		//! Add other statistics.
		ERRVALUE Add (
			const STAT_MULTIVAR& other		//!< Other stats to add
			);

		//! Compute eigenvectors, eigenvalues, correspondences.
		ERRVALUE ComputeEigen ();

		//! Get correlation matrix.
		const MIMATRIX& GetCorrelation () const
			{ return (m_Correlation); }

		//! Get matrix of correspondence between input variable and principal component.
		//! Must call ComputeEigen before using this method.
		const MIMATRIX& GetCorrespondence () const
			{ return (m_Correspondence); }

		//! Get covariance matrix.
		const MIMATRIX& GetCovariance () const
			{ return (m_Covariance); }

		//! Get eigenvalues matrix.
		//! Must call ComputeEigen before using this method.
		const MIMATRIX& GetEigenValues () const
			{ return (m_EigenValues); }

		//! Get eigenvectors matrix.
		//! Must call ComputeEigen before using this method.
		const MIMATRIX& GetEigenVectors () const
			{ return (m_EigenVectors); }

		//! Compute multilinear regression coefficients for specified target variable.
		//! The coefficients are returned in a vector MIMATRIX with (NumSrcVars+1) elements.
		//! The first element (index = 0) will be the "offset" value.  Subsequent elements
		//! (starting with index = 1) correspond to the source variables in sequence.
		//! Must call Finish() before using or error will result.
		ERRVALUE ComputeMultilinearRegression (
			MIMATRIX& CoeffRet,					//!< Coefficients returned
			UINT32 TgtVar,							//!< Target variable index
			const UINT32 *SrcVars = 0,			//!< Source variables, 0 to use all except specified target
			UINT32 NumSrcVars = 0				//!< Number of source variables, 0 to use all except specified target
			) const;

		//! Get maximum values.
		const double* GetMaximums () const
			{ return (m_Maximum); }

		//! Get mean values.
		const double* GetMeans () const
			{ return (m_Mean); }

		//! Get minimum values.
		const double* GetMinimums () const
			{ return (m_Minimum); }

		//! Get number of samples.
		UINT64 GetNumSamples () const
			{ return (m_NumSamples); }

		//! Finish accumulation, compute mean, std deviation, correlation, covariance.
		ERRVALUE Finish ();

		//! Start accumulation, resets all statistics.
		ERRVALUE Start (
			UINT32 NumVars							//!< Number of variables
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		enum STATE {
			STATE_Invalid,
			STATE_Accumulating,
			STATE_Finished
			};

		MIMATRIX m_Sum2;
		MIMATRIX m_Covariance;
		MIMATRIX m_Correlation;
		MIMATRIX m_EigenVectors;
		MIMATRIX m_EigenValues;
		MIMATRIX m_Correspondence;					// Correspondence between variable and principal component
		DOUBLE_ARRAY<double> m_Minimum;
		DOUBLE_ARRAY<double> m_Maximum;
		DOUBLE_ARRAY<double> m_Mean;
		DOUBLE_ARRAY<double> m_Sum;
		UINT64 m_NumSamples;
		UINT32 m_NumVars;
		STATE m_State;
		bool m_HasEigen;

		ERRVALUE ComputeBasicStats ();
		void Copy (const STAT_MULTIVAR& rhs);

		#endif	// GENERATING_DOXYGEN_OUTPUT
	};

#endif	// INC_MI32_STATMULTIVAR_H

