/***
 *	\file transmod.h <mi32/transmod.h>
 *	\brief Definitions for TransModel object used in georeference
 *
 * \if NODOC
 * $Id: transmod.h_v 1.9 2011/09/23 17:49:07 mju Exp $
 *
 * $Log: transmod.h_v $
 * Revision 1.9  2011/09/23 17:49:07  mju
 * Use trans2dmodel enum.
 *
 * Revision 1.7  2004/07/30 17:43:41  mju
 * Add getName.
 *
 * Revision 1.5  2004/01/20 17:47:09  vdronov
 * new FLAG_3D added and Is3D method as well
 *
 * Revision 1.2  2000/11/16 17:50:10  scowan
 * \endif
***/

#ifndef	INC_MI32_TRANSMOD_H
#define	INC_MI32_TRANSMOD_H

#ifndef INC_MI32_TRANS2DMODEL_H
#include <mi32/trans2dmodel.h>
#endif

class MISTRING;


//! Manages the different sections and values in the TRANS2DMODEL enum.
//! Its main goal is to hide the bit twiddling that the TRANS2DMODEL variable requires.
class TRANSMODEL {
	public:

		enum FLAGS {							//! Translation model flags
			FLAGS_None =				0,
			FLAGS_OrientInvert = 	TRANS2DMODEL_OrientInvert,
			FLAGS_Piecewise = 		TRANS2DMODEL_Piecewise,
			FLAGS_3D = 					TRANS2DMODEL_3D
			};

		//! Translation model type, based off of TRANS2DMODEL values
		enum TYPE {
			TYPE_None = 0,
			TYPE_Affine = 1,						//!< Affine
			TYPE_PlaneProj,							//!< Plane-projective
			TYPE_Bilinear,
			TYPE_Polynomial,
			TYPE_Quintic,
			TYPE_SpaceResect,						//!< Space resection (aerial photography)
			TYPE_Conformal,							//!< Conformal transformation
			TYPE_RubberSheet,						//!< "Rubber Sheeting" or "Morphing" model
			TYPE_RationalPolynomial
			};

		//! Default constructor	
		TRANSMODEL (
			):
			m_Order(0),
			m_Flags(FLAGS_None),
			m_Type(TYPE_None)
			{ }

		//! Construct using TRANS2DMODEL.
		TRANSMODEL (
			TRANS2DMODEL model
			) { SetModel(model); }

		//! Construct using legacy UINT16 model.
		TRANSMODEL (
			UINT16 model
			) { SetModel(model); }

		//! Constructor with type
		TRANSMODEL (
			TYPE dfttype
			) :
			m_Order(0),
			m_Flags(FLAGS_None),
			m_Type(dfttype)
			{ }
			
		//! Obtain TRANSMODEL flags.
		//! @return: TRANSMODEL::FLAGS.
		FLAGS GetFlags (
			) const { return (m_Flags); }

		//! Obtain old translation model combined value.
		//! @return: Translation model combined value.
		TRANS2DMODEL GetModel () const
			{ return (static_cast<TRANS2DMODEL>(m_Flags | (m_Type << 8) | m_Order)); }

		//! Get model name.
		void GetName (
			MISTRING& name
			) const;

		//! Obtain translation model order.
		//! @return: Translation model order.
		UINT16 GetOrder (
			) const { return (m_Order); }

		//! Obtain translation model type.
		//! @return: Translation model type.
		TYPE GetType (
			) const { return (m_Type); }

		//! Determine if translation model is 3D.
		//! @return: True if translation model is 3D, false if not.
		bool Is3D (
			) const { return ((m_Flags & FLAGS_3D) != 0); }

		//! Determine translation model orientation.
		//! @return: True if translation model inverted, false if not
		bool IsOrientInvert (
			) const { return ((m_Flags & FLAGS_OrientInvert) != 0); }

		//! Determine if translation model is piecewise.
		//! @return: True if translation model is piecewise, false if not
		bool IsPiecewise (
			) const { return ((m_Flags & FLAGS_Piecewise) != 0); }

		//! Set translation model TRANS2DMODEL value.
		void SetModel (
			TRANS2DMODEL model
			) {
			m_Order = static_cast<UINT16>(model & 0x00FF),
			m_Flags = static_cast<FLAGS>(model & 0xF000);
			m_Type = static_cast<TYPE>((model & 0xF00) >> 8);
			}

		//! Set translation model using legacy UINT16 value.
		void SetModel (
			UINT16 model
			) { SetModel(static_cast<TRANS2DMODEL>(model)); }

		//! Set translation model flags
		void SetFlags (
			FLAGS flags
			) { m_Flags = flags; }

		//! Set translation model order
		void SetOrder (
			UINT16 order
			) { m_Order = order; }

		//! Set translation model type
		void SetType (
			TYPE type
			) { m_Type = type; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FLAGS m_Flags;
		TYPE m_Type;
		UINT16 m_Order;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(TRANSMODEL::FLAGS);
#endif

#endif					// INC_MI32_TRANSMOD_H 
