/**
 * \file rvc/opprofiling.h
 * \brief Definitions for RVC::OP::PROFILING class.
 *
 * \if NODOC
 * $Id: opprofiling.h_v 1.2 2011/10/13 21:12:01 mju Exp $
 *
 * $Log: opprofiling.h_v $
 * Revision 1.2  2011/10/13 21:12:01  mju
 * Remove redundant hdr.
 *
 * Revision 1.1  2006/11/30 17:08:42  vdronov
 * Initial revision
 * \endif
**/

#ifndef  INC_RVC_OPPROFILING_H
#define  INC_RVC_OPPROFILING_H

#ifndef  INC_POLYLINE_H
#include <mi32/polyline.h>
#endif

#ifndef  INC_ELEMSTYL_H
#include <mi32/elemstyl.h>
#endif

namespace RVC {

#ifndef GENERATING_DOXYGEN_OUTPUT
class OBJITEM;
#endif // GENERATING_DOXYGEN_OUTPUT

namespace OP {

   //! Class for creating profiles.
class PROFILING {
public:

	enum METHOD {
		METHOD_Invalid = -1,
		METHOD_First = 0,
		METHOD_StackProfiles = 0,     
		METHOD_Last = 0,
		METHOD_Count = 1
		};

	//! Class descrines parameters of destination object.
	class DESTINATION_PARAMETERS {
	public:
		POLYSTYLE m_PolygonStyle;
		LINESTYLE m_LineStyle;
		};

	//! Base class for all profiling methods.
	class METHOD_PARAMETERS {
	public:

		METHOD_PARAMETERS (
			const METHOD method
			) : m_Method(Check(method)) {}
		virtual ~METHOD_PARAMETERS (
			) {}

		//! Return enum of method in use
		METHOD GetMethod (
			) const { return m_Method; };

	private:

#ifndef GENERATING_DOXYGEN_OUTPUT

		const METHOD m_Method;

		METHOD_PARAMETERS (); // not implemented

		static METHOD Check (
			const METHOD method
			) {
			if (method < METHOD_First) return METHOD_Invalid;
			if (method > METHOD_Last) return METHOD_Invalid;
			return method;
			}

#endif // GENERATING_DOXYGEN_OUTPUT

		};

   //! These parameters describes stack profiles of surface represented by raster 
	//! build by static method ProcessStackProfiles. DistanceAcross specifies 
	//! distance in cells between profiles and must be in range (1, 8192) and 
	//! default is 12. DistanceAlong specifies distance in cells between sample 
	//! points on profile and must be in range (1, 8192) and default is 4. 
	//! MaximumAmplitude specifies maximum amplitude of profile and must be in 
	//! range (1, 8192) and default is 32. RotationAngle specifies directional 
	//! angle of profile in degrees and must be in range (-90.0, 90.0) and default 
	//! is  0 which is direction to East in raster coordinates. SplineType specifies
	//! spline type of profile line and can be None, Quadratic or Qubic BSpline and 
	//! default is None. ShowBaselines specifies displaying baseline of profiles 
	//! that lies on level of middle of range of surface values and default is false. 
	//! ShowAreaAbove specify displaying area above baselines by drawing verical lines
	//! there and default is false.
	class STACKPROFILES_PARAMETERS : public METHOD_PARAMETERS {
	public:

		STACKPROFILES_PARAMETERS (
			) :
			METHOD_PARAMETERS(METHOD_StackProfiles),
			m_DistanceAcross(12),
			m_DistanceAlong(4),
			m_MaximumAmplitude(32),
			m_RotationAngle(0.0),
			m_SplineType(POLYLINE::SPLINE_NoSpline),
			m_ShowBaselines(false),
			m_ShowAreaAbove(false)
			{}

			~STACKPROFILES_PARAMETERS () {}

		//! See class comment
		INT32 GetDistanceAcross (
			) const { return m_DistanceAcross; }

		//! See class comment
		INT32 GetDistanceAlong (
			) const { return m_DistanceAlong; }

		//! See class comment
		INT32 GetMaximumAmplitude (
			) const { return m_MaximumAmplitude; }

		//! See class comment
		double GetRotationAngle (
			) const { return m_RotationAngle; }

		//! See class comment
		bool GetShowAreaAbove (
			) const { return m_ShowAreaAbove; }

		//! See class comment
		bool GetShowBaselines (
			) const {return m_ShowBaselines; }

		POLYLINE::SPLINE GetSplineType (
			) const {return m_SplineType; }

		//! See class comment
		INT32 SetDistanceAcross (
			const INT32 distance
			) { m_DistanceAcross = bound(distance, 1, 8192); return m_DistanceAcross; }

		//! See class comment
		INT32 SetDistanceAlong (
			const INT32 distance
			) { m_DistanceAlong = bound(distance, 1, 8192); return m_DistanceAlong; }

		//! See class comment
		INT32 SetMaximumAmplitude (
			const INT32 amplitude
			) { m_MaximumAmplitude = bound(amplitude, 1, 8192); return m_MaximumAmplitude; }

		//! See class comment
		double SetRotationAngle (
			const double angle
			) { m_RotationAngle = bound(angle, -90.0, 90.0); return m_RotationAngle; }

		//! See class comment
		void SetShowAreaAbove (
			const bool show
			) { m_ShowAreaAbove = show; }

		//! See class comment
		void SetShowBaselines (
			const bool show
			) { m_ShowBaselines = show; }

		//! See class comment
		void SetSplineType (
			const POLYLINE::SPLINE spline
			) { m_SplineType = spline; }

	private:

#ifndef GENERATING_DOXYGEN_OUTPUT

		INT32 m_DistanceAcross;
		INT32 m_DistanceAlong;
		INT32 m_MaximumAmplitude;
		double m_RotationAngle;

		POLYLINE::SPLINE m_SplineType;

		bool m_ShowBaselines;
		bool m_ShowAreaAbove;

#endif // GENERATING_DOXYGEN_OUTPUT

		};

      //! Generates profiles
      static ERRVALUE Process (
         const RVC::OBJITEM& SrcObjItem,
         RVC::OBJITEM& DestObjItem, 
			const METHOD_PARAMETERS* MethodParameters,
			const DESTINATION_PARAMETERS& DestParameters 
         );

       //! Generates cad stack profiles from raster
      static ERRVALUE ProcessStackProfiles (
         const RVC::OBJITEM& SrcObjItem,
         RVC::OBJITEM& DestObjItem, 
			const STACKPROFILES_PARAMETERS& MethodParameters,
			const DESTINATION_PARAMETERS& DestParameters 
         );
        
};

}  // end namespace OP
}  // end namespace RVC


#endif  // INC_RVC_OPPROFILING_H
