/******************************************************************************
 *
 * \file sml/sml.h
 * \brief SML string class
 *
 * \if NODOC
 * $Log: string.h_v $
 * Revision 1.4  2008/10/23 20:38:13  scowan
 * Moved function prototypes from sml include file.
 *
 * Revision 1.3  2007/08/02 20:42:07  dwilliss
 * const
 *
 * Revision 1.2  2007/05/08 15:46:22  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 20:44:34  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_STRING_H
#define INC_SML_STRING_H

#ifndef INC_MI32_MISTRING_H
#include <mi32/mistring.h>
#endif

#ifndef INC_MI32_USERDATA_H
#include <sml/userdata.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

namespace SML {
union ARG;
class FLAG;
class PARM;

/***************************************************************************/
//! \class SML::STRING sml/string.h
//! \brief SML string constant representation
//!
//! \note  The constructors are private. Use the various
//! SML::CONTEXT::StringAlloc() methods instead.
class RVCSYSLIBCLASSEXPORT STRING : public MISTRING {

	#if defined (RVCSYSDLL)
	public:
	#else
	private:
	#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
	//! Constructors are private outside the DLL

	//! Constructor for empty string.
	STRING (
		);

	//! Constructor for ASCII (ISO-2022) string.
	//! Direct use of constructor outside rvcsys.dll deprecated
	//! use context->StringAlloc() instead, since the application
	//! doesn't know the size of an SML::STRING.
	STRING (
		const char *s
		);

	//! Constructor for any encoding SML::STRING
	//! Direct use of constructor outside rvcsys.dll deprecated
	//! use context->StringAlloc() instead, since the application
	//! doesn't know the size of an SML::STRING.
	STRING (
		const void *s,
		CHAR_ENCODING encoding
		);

	//! Constructor for Unicode string.
	//! Direct use of constructor outside rvcsys.dll deprecated
	//! use context->StringAlloc() instead, since the application
	//! doesn't know the size of an SML::STRING
	STRING (
		const MIUNICODE *s
		);

	//! Constructor for UTF8 string.
	//! Direct use of constructor outside rvcsys.dll deprecated
	//! use context->StringAlloc() instead, since the application
	//! doesn't know the size of an SML::STRING
	STRING (
		const UINT8 *s
		);

	//! Copy constructor
	STRING (
		const STRING& rhs
		);

private:
	
	//! Destructor.
	//! Can only destroy via a call to Release()
	virtual ~STRING();

#endif // GENERATING_DOXYGEN_OUTPUT
public:

	//! Attach some User Data to the string.
	//! The data to be attached must be derived from SML::USERDATA, and must have
	//!	been allocated via "new".  Once attached to the string, the string owns the
	//! data and will "delete" it when the string is deleted.
	//! Also, if you try to attach something for a user ID that's already in use,
	//! it will delete the existing thing.
	void AttachUserData (
		USERDATA* data
		);

	//! Detach some User Data from the string.
	//! If the data to be unattached isn't attached, it just ignores you.
	//! Once detached, the data is yours to do with as you wish.  You are
	//! responsible for deleting it too.
	void DetachUserData (
		USERDATA* data
		);


	//! Get the User Data attached to a string.
	//! If no data for the given id is attached, it will just return NULL	.
	//! Do not free the data returned unless you Unattach it first.
	USERDATA* GetUserData (
		USERDATA::TAG tag
		);

	//! Increment refcount.
	void AddRef ();

	//! Decrement refcount.  If it goes to zero, string deletes itself.
	void Release ();

	//! Lookup a string value in an array of SML::FLAGS.
	//!
	//! In an SML function, a parameter may be defined to be STYPE_String
	//! even though internally you want a number.  This function allows you
	//! to look up the value in an array of SML::FLAGs
	//!
	//! @return -1 if the string could not be found in the SML::FLAG array,
	//! 0 if it could
	int FlagTextToValueSingle (
		const FLAG *flags, 		//!< SML::FLAG array describing possible values
		UINT32 *FlagVal		//!< flag value returned
		) const;

	//! Lookup a string value in an array of SML::FLAGS.
	//!
	//! This method is just like the other one of the same name, but takes
	//! an SML::PARM and gets the SML::FLAG array from it instead.
	int FlagTextToValueSingle (
		const PARM *flags, 		//!< an SML::PARM which contains an SML::FLAG array
		UINT32 *FlagVal		//!< flag value returned
		) const;

	//! Lookup multiple string values in an array of SML::FLAGS.
	//!
	//! This method is like FlagTextToValueSingle except that the string
	//! may contain multiple flags seperated by commas and/or whitespace.
	//! This is intended for flags which are bit-field values that you would
	//! use bitwise or to combine in C or C++
	//!
	//! @return -1 if the any of the flags could not be found in the SML::FLAG
	//! array, 0 if all were found
	int FlagTextToValueMultiple (
		const FLAG *flags, 		//!< SML::FLAG array describing possible values
		UINT32 *FlagVal		//!< flag value returned
		) const;

	//! Lookup multiple string values in an array of SML::FLAGS.
	//!
	//! This method is just like the other one of the same name, but takes
	//! an SML::PARM and gets the SML::FLAG array from it instead.
	int FlagTextToValueMultiple (
		const PARM *flags, 		//!< an SML::PARM which contains an SML::FLAG array
		UINT32 *FlagVal		//!< flag value returned
		) const;

	//! Lookup a flag value in an array of SML::FLAGS.
	//!
	//! In an SML function, a parameter may be defined to be STYPE_String
	//! even though internally you have a number.  This function allows you
	//! to look up the string in an array of SML::FLAGs
	//!
	//! @return -1 if the string could not be found in the SML::FLAG array,
	//! 0 if it could
	int FlagValueToTextSingle (
		const FLAG *flags, 		//!< SML::FLAG array describing possible values
		UINT32 *FlagVal		//!< flag value returned
		);

	//! Lookup a flag value in an array of SML::FLAGS.
	//!
	//! This method is just like the other one of the same name, but takes
	//! an SML::PARM and gets the SML::FLAG array from it instead.
	int FlagValueToTextSingle (
		const PARM *flags, 		//!< an SML::PARM which contains an SML::FLAG array
		UINT32 *FlagVal		//!< flag value returned
		);

	//! Lookup multiple flag values in an array of SML::FLAGS.
	//!
	//! This method is like FlagTextToValueSingle except that the string
	//! may contain multiple flags seperated by commas and/or whitespace.
	//! This is intended for flags which are bit-field values that you would
	//! use bitwise or to combine in C or C++
	//!
	//! @return -1 if the any of the flags could not be found in the SML::FLAG
	//! array, 0 if all were found
	int FlagValueToTextMultiple (
		const FLAG *flags, 		//!< SML::FLAG array describing possible values
		UINT32 *FlagVal		//!< flag value returned
		);

	//! Lookup multiple flag values in an array of SML::FLAGS.
	//!
	//! This method is just like the other one of the same name, but takes
	//! an SML::PARM and gets the SML::FLAG array from it instead.
	int FlagValueToTextMultiple (
		const PARM *flags, 		//!< an SML::PARM which contains an SML::FLAG array
		UINT32 *FlagVal		//!< flag value returned
		);

#ifndef GENERATING_DOXYGEN_OUTPUT
#ifdef RVCSYSDLL
	// Old versions which we need to export for a while (until about 6 Aug 07)
	// but don't want other code to use yet
	int FlagTextToValueSingle (
		FLAG *flags, 		//!< SML::FLAG array describing possible values
		UINT32 *FlagVal		//!< flag value returned
		);

	int FlagTextToValueSingle (
		PARM *flags, 		//!< an SML::PARM which contains an SML::FLAG array
		UINT32 *FlagVal		//!< flag value returned
		);

	int FlagTextToValueMultiple (
		FLAG *flags, 		//!< SML::FLAG array describing possible values
		UINT32 *FlagVal		//!< flag value returned
		);

	int FlagTextToValueMultiple (
		PARM *flags, 		//!< an SML::PARM which contains an SML::FLAG array
		UINT32 *FlagVal		//!< flag value returned
		);

	int FlagValueToTextSingle (
		FLAG *flags, 		//!< SML::FLAG array describing possible values
		UINT32 *FlagVal		//!< flag value returned
		);

	int FlagValueToTextSingle (
		PARM *flags, 		//!< an SML::PARM which contains an SML::FLAG array
		UINT32 *FlagVal		//!< flag value returned
		);

	int FlagValueToTextMultiple (
		FLAG *flags, 		//!< SML::FLAG array describing possible values
		UINT32 *FlagVal		//!< flag value returned
		);

	int FlagValueToTextMultiple (
		PARM *flags, 		//!< an SML::PARM which contains an SML::FLAG array
		UINT32 *FlagVal		//!< flag value returned
		);
#endif
#endif

	//! Get STRUTF8 instance for string.
	//! @return Temporary STRUTF8 instance.
	DEPRECATED_MSG("Just construct a STRUTF8 from *smlstr") STRUTF8 GetSTRUTF8 (
		) const;

	//! (char*) version assumes ISO-2022.
	DEPRECATED_MSG("Use Assign instead") void SetString (
		const char* str
		);

private:
#ifndef GENERATING_DOXYGEN_OUTPUT
	int m_refcount;
	USERDATACONTAINER m_userdata;

	friend class CONTEXT;	// to shut up VC7 warning about having private dtor and no friends.  Dtor is called by DecRefCound when refcount is decremented to 0.
#endif // GENERATING_DOXYGEN_OUTPUT
	};

//! Internal function for implementing SML's printf()-like functions
RVCSYSLIBEXPORT void	PrintfSubMISTRING (
	MISTRING& output, //!< String to print into
	int numargs, 		//!< number of arguments
	SML::ARG *args, 		//!< The printf arguments (numargs of them)
	UINT8 *type			//!< the SML::STYPEs of the args
	);
	
} // end of SML namespace

typedef SML::STRING SMLSTRING;
#define _SML_PrintfSubMISTRING SML::PrintfSubMISTRING

#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT

#endif		//  INC_SML_STRING_H


