/**
 * \file imagetargetfile.h <spatmod/imagetargetfile.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_FILE definition
 *
 * \if NODOC
 * $Id: imagetargetfile.h_v 1.20 2013/06/25 17:22:10 mju Exp $
 *
 * $Log: imagetargetfile.h_v $
 * Revision 1.20  2013/06/25 17:22:10  mju
 * Use mistatuscontext.
 *
 * Revision 1.19  2009/08/13 16:45:58  mju
 * Make factory non-nested.
 *
 * Revision 1.18  2009/08/13 15:37:51  mju
 * Add geoFormats to factory.
 *
 * Revision 1.17  2009/08/13 14:49:02  mju
 * Add factory class.
 *
 * Revision 1.16  2007/07/10 13:33:17  mju
 * Add statusContext member.
 *
 * Revision 1.15  2007/06/25 14:43:04  mju
 * Add m_didTileBegin.
 *
 * Revision 1.14  2007/03/20 22:11:19  dfriberg
 * Allow specification of tile size in TILEPARMS
 *
 * Revision 1.13  2007/01/26 23:26:38  dfriberg
 * Add virtual methods to be called when writing one output file per tile
 *
 * Revision 1.12  2006/12/13 22:13:14  dfriberg
 * Fix incorrectly named delegate
 *
 * Revision 1.11  2006/12/11 21:00:35  mju
 * Add delegate for "tile end" and notifyTileEnd.
 *
 * Revision 1.10  2006/12/08 18:58:29  mju
 * Revise design for tiled output to have single delegate.
 *
 * Revision 1.9  2006/12/07 22:46:53  mju
 * Add methods for using memory buffer.
 * Add initial design for tiling to separate files.
 *
 * Revision 1.8  2006/06/08 17:08:17  mju
 * Add queryGeoFormat.
 *
 * Revision 1.7  2006/05/05 21:20:41  mju
 * Add GEOFORMAT_ForceKML.
 *
 * Revision 1.6  2006/03/09 17:17:49  mju
 * Add fn to save kml.
 *
 * Revision 1.5  2006/03/08 18:52:10  mju
 * Define geoformat_kml and all.
 *
 * Revision 1.4  2005/12/16 16:08:46  mju
 * Use bits for geoformats.
 *
 * Revision 1.3  2005/12/16 16:07:48  mju
 * Change geoformats to allow multiple.
 *
 * Revision 1.2  2005/12/15 23:19:02  mju
 * Include filepath hdr.
 *
 * Revision 1.1  2005/12/09 17:59:16  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETFILE_H
#define	INC_SPATMOD_IMAGETARGETFILE_H

#ifndef	INC_SPATMOD_IMAGETARGET_H
#include <spatmod/imagetarget.h>
#endif

#ifndef	INC_SPATMOD_IMAGEFORMAT_H
#include <spatmod/imageformat.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef	INC_MI32_DELEGATE_H
#include <mi32/delegate.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class MISTATUSCONTEXT;
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Base for target file.
class TARGET_FILE : public TARGET {
	public:

		//! Georeference format enumeration.
		enum GEOFORMATS {
			GEOFORMAT_None =			0x0000,
			GEOFORMAT_ArcWorld =		0x0001,
			GEOFORMAT_KML =			0x0002,
			GEOFORMAT_AUX =			0x0004,
			GEOFORMAT_ALL =			0x00FF,
			GEOFORMAT_ForceKML =		0x8000	//!< Force to write KML even if not LatLon georeference
			};

		//! Parameters for control of output to separate file for each tile.
		class TILEPARMS {
			public:

				//! Constructor.
				TILEPARMS ();

				//! Delegate for adding new tile.
				//! Parameters are tile column, tile row, path, extent in overall image coordinates
				//! This will not be called for tiles having no valid pixels.
				typedef fastdelegate::FastDelegate<ERRVALUE(INT32 TileColumn, INT32 TileRow, FILEPATH&, const LRECT2D& extent)> DELEGATE_ONTILEADD;

				//! Set whether to auto-trim tile to minimum valid extent.
				//! Default is to output full-extent tiles except when completely empty of valid pixels.
				//! Not supported by all formats or tilings, will behave as default if not supported.
				void SetAutoTrim (
					bool AutoTrim = true
					) { m_AutoTrim = AutoTrim; }

				//! Set delegate to call when adding new tile.
				void SetDelegateOnTileAdd (
					DELEGATE_ONTILEADD delegate
					) { m_DelegateOnTileAdd = delegate; }

				//! Set delegate to call when finished writing tile.
				void SetDelegateOnTileEnd (
					DELEGATE_ERRVALUE_NOPARMS delegate
					) { m_DelegateOnTileEnd = delegate; }

				void SetTileDimensions (
					double ColumnsPerTile,
					double RowsPerTile
					) { m_ColumnsPerTile = ColumnsPerTile; m_RowsPerTile = RowsPerTile; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				DELEGATE_ONTILEADD m_DelegateOnTileAdd;
				DELEGATE_ERRVALUE_NOPARMS m_DelegateOnTileEnd;
				bool m_AutoTrim;
				double m_ColumnsPerTile;
				double m_RowsPerTile;

				friend class IMAGE::TARGET_FILE;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Construct with specified input stage and path to target file.
		//! Target does not take ownership of input stage.
		TARGET_FILE (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			const FILEPATH& filepath,					//!< Path to target file
			GEOFORMATS GeoFormats = GEOFORMAT_None	//!< Georeference formats
			);

		//! Construct with specified input stage.
		//! Target does not take ownership of input stage.
		TARGET_FILE (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			GEOFORMATS GeoFormats = GEOFORMAT_None	//!< Georeference formats
			);

		//! Destructor.
		virtual ~TARGET_FILE ();

		//! Detach memory buffer.
		//! Caller is responsible for freeing buffer.
		//! @return Pointer to buffer or NULL if writing to file.
		void* DetachBuffer (
			) { return (v_DetachBuffer()); }

		//! Get pointer to memory buffer.
		//! @return Pointer to buffer or NULL if writing to file.
		const void* GetBuffer (
			) const { return (v_GetBuffer()); }

		//! Get memory buffer size.
		//! @return Number of bytes in buffer or 0 if no memory buffer used.
		INT64 GetBufferSize (
			) const { return (v_GetBufferSize()); }

		//! Get path to output file.
		const FILEPATH& GetFilePath (
			) const { return (m_filepath); }

		//! Get preferred georeference format.
		GEOFORMATS GetGeoFormats (
			) const { return (m_GeoFormats); }

		//! Determine if is to output one tile per file.
		bool IsOneTilePerFile (
			) const { return (m_IsOneTilePerFile); }

		//! Determine if specified georeference format can be written.
		//! @return true if can be written, false if not.
		//! May return true for partial support in which case message will contain limitation.
		//! If not supported message will contain explanation.
		bool QueryGeoFormat (
			GEOFORMATS GeoFormat,						//!< Single GEOFORMAT value to check
			MISTRING& message								//!< Message string returned containing limitation reason not supported
			) const;

		//! Set tiling parameters for output of one "file" per tile.
		//! If used, must be called before Initialize().
		void SetTileParms (
			const TILEPARMS& TileParms
			);

	protected:

		//! Write georeference formats (if available from source).
		//! If no georeference is available from source, no error will be returned.
		//! Error will be returned if cannot convert source georeference to current GeoFormat.
		ERRVALUE WriteGeoFormats ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		FILEPATH m_filepath;
		GEOFORMATS m_GeoFormats;
		bool m_IsOneTilePerFile;
		bool m_DidTileBegin;
		TILEPARMS m_TileParms;
		LRECT2D m_CurTileExtents;
		MISTATUSCONTEXT *m_pStatusContext;

		//! Notify when adding new tile.
		//! May not be called until have a known valid pixel in the tile.  May never be called
		//! if all pixels in the tile are invalid.  Must always and only be called if tiling to separate files.
		//! Will write georeference formats for tile in formats specified in constructor.
		ERRVALUE NotifyTileAdd (
			INT32 TileColumn,								//!< Tile column
			INT32 TileRow,									//!< Tile row
			FILEPATH& filepath,							//!< Returned file path to write
			LRECT2D& extents								//!< Extents passed/returned, if initially invalid/empty will be computed as full tile
			);

		//! Notify when ending.
		//! Must always and only be called if NotifyTileAdd was previously called for the current tile.
		ERRVALUE NotifyTileEnd (
			);

		//! Write georeference formats (if available from source).
		//! If no georeference is available from source, no error will be returned.
		//! Error will be returned if cannot convert source georeference to current GeoFormat.
		ERRVALUE WriteGeoFormats (
			const FILEPATH& filepath,					//!< Path to image file
			const LRECT2D& extent						//!< Extent file applies to in overall image coordinates
			) const;

		ERRVALUE WriteGeoArcWorld (const FILEPATH& filepath, const DRECT2D& extent) const;
		ERRVALUE WriteGeoKML (const FILEPATH& filepath, const DRECT2D& extent) const;

		// Unimplemented
		TARGET_FILE (const TARGET_FILE&);
		TARGET_FILE& operator= (const TARGET_FILE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

		//! Detach memory buffer.
		//! Caller is responsible for freeing buffer.
		virtual void* v_DetachBuffer (
			);

		//! Derived class should write null data to the output file for the number of
		//! rows specified.  Will only be called when writing one file per tile.
		virtual ERRVALUE v_FillTileRows (
			INT32 NumColumns,
			INT32 NumRows
			) = 0;

		//! Get pointer to memory buffer.
		//! @return Pointer to buffer or NULL if writing to file.
		virtual const void* v_GetBuffer (
			) const;

		//! Get memory buffer size.
		virtual INT64 v_GetBufferSize (
			) const;

		//! Final - derived classes do not implement this.
		ERRVALUE v_PushSubSection (const SECTION& SubSection, const BUFFER& buffer);

		//! Called only when writing one file per tile.  Derived classes should open
		//! file here and do any required preprocessing.
		virtual ERRVALUE v_TileBegin (
			const FILEPATH& FilePath,
			const LRECT2D& Extents
			) = 0;

		//! Called only when writing one file per tile.  Derived classes should close
		//! file here and do any required postprocessing.
		virtual ERRVALUE v_TileEnd (
			) = 0;

		//! Write a subsection to the output file.  Called by v_PushSubSection.
		virtual ERRVALUE v_WriteSubSection (
			const SECTION& SubSection,
			const BUFFER& buffer
			) = 0;

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(TARGET_FILE::GEOFORMATS);
#endif

//=====================================================================================================================

//! Base interface factory class for constructing TARGET_FILE instance.
class FACTORY_TARGET_FILE {
	public:

		//! Destructor.
		virtual ~FACTORY_TARGET_FILE ();

		//! Clone this FACTORY instance.
		FACTORY_TARGET_FILE * Clone (
			) const { return (v_Clone()); }

		//! Construct TARGET_FILE instance.
		TARGET_FILE * ConstructTarget (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			const FILEPATH& filepath					//!< Path to target file
			) const { return (v_ConstructTarget(pInputStage,filepath,m_GeoFormats)); }

		//! Construct TARGET_FILE instance with specified TARGET_FILE::GEOFORMATS.
		TARGET_FILE * ConstructTarget (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			const FILEPATH& filepath,					//!< Path to target file
			TARGET_FILE::GEOFORMATS GeoFormats						//!< Georeference formats
			) const { return (v_ConstructTarget(pInputStage,filepath,GeoFormats)); }

		//! Get enumerated format specification.
		FORMAT::SPEC GetFormatSpec (
			) const { return (m_FormatSpec); }

		//! Set georeference formats to use when constructing target.
		void SetGeoFormats (
			TARGET_FILE::GEOFORMATS GeoFormats
			) { m_GeoFormats = GeoFormats; }

	protected:

		//! Standard constructor.
		explicit FACTORY_TARGET_FILE (FORMAT::SPEC FormatSpec);

		//! Copy constructor.
		FACTORY_TARGET_FILE (const FACTORY_TARGET_FILE& rhs);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FORMAT::SPEC m_FormatSpec;
		TARGET_FILE::GEOFORMATS m_GeoFormats;

		// Unimplemented.
		FACTORY_TARGET_FILE& operator= (const FACTORY_TARGET_FILE& rhs);
		#endif

		//! Clone factory instance.
		virtual FACTORY_TARGET_FILE * v_Clone () const = 0;

		//! Construct TARGET_FILE instance.
		virtual TARGET_FILE * v_ConstructTarget (
			STAGE *pInputStage,							//!< Stage in pipeline to obtain image data from
			const FILEPATH& filepath,					//!< Path to target file
			TARGET_FILE::GEOFORMATS GeoFormats						//!< Georeference formats
			) const = 0;

	};

//=====================================================================================================================
}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETFILE_H
